#include <apsrlib/apsrlib.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

void parsePacket	(xmlDocPtr, xmlNodePtr);
void parseInfo		(xmlDocPtr, xmlNodePtr);
void parseEthernet	(xmlDocPtr, xmlNodePtr);
void parseARP		(xmlDocPtr, xmlNodePtr);

int main (int argc, char **argv) {
	char *docname;
	xmlDocPtr doc;
	xmlNodePtr cur;

	if(argc != 2)
		ERROR_EXIT("Usage: parseit <file.xml>");
	
	docname = xmalloc(strlen(argv[1]));
	strncpy(docname, argv[1], strlen(argv[1]));
	
	doc = xmlParseFile(docname);

	if(doc == NULL)
		ERROR_EXIT("Document NOT parsed successfully!\n");

	cur = xmlDocGetRootElement(doc);

	if(cur == NULL) {
		ERROR("Document is empty!\n");
		xmlFreeDoc(doc);
		exit(EXIT_FAILURE);
	}

	if(xmlStrcmp(cur->name, (const xmlChar *) "packet")) {
		ERROR("Document of the wrong type, root != packet!\n");
		xmlFreeDoc(doc);
		exit(EXIT_FAILURE);
	}

	cur = cur->xmlChildrenNode;
	
	while(cur != NULL) {
		if((!xmlStrcmp(cur->name, (const xmlChar *) "packet"))) {
			parsePacket (doc, cur);
		}

		cur=cur->next;
	}

	exit(EXIT_SUCCESS);
}


void parsePacket (xmlDocPtr doc, xmlNodePtr cur) {
	cur = cur->xmlChildrenNode;

	while(cur != NULL) {
		if((!xmlStrcmp(cur->name, (const xmlChar *) "info"))) {
			parseInfo(doc, cur);
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "ethernet"))) {
			parseEthernet(doc, cur);
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "arp"))) {
			parseARP(doc, cur);
		}

		cur=cur->next;
	}
}

void parseInfo (xmlDocPtr doc, xmlNodePtr cur) {
	cur = cur->xmlChildrenNode;

	while(cur != NULL) {
		if((!xmlStrcmp(cur->name, (const xmlChar *) "arrival"))) {
			apsr_output("\n%s - Received packet",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "length"))) {
			apsr_output(" (%s bytes length)...\n\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		cur = cur->next;
	}
}
		

void parseEthernet (xmlDocPtr doc, xmlNodePtr cur) {
	cur = cur->xmlChildrenNode;
	apsr_output("- Ethernet Header -\n");

	while(cur != NULL) {
		if((!xmlStrcmp(cur->name, (const xmlChar *) "destination"))) {
			printf("  MAC Dst: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "source"))) {
			printf("  MAC Src: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "typeII"))) {
			printf("  Type (Ethernet II): %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		cur=cur->next;
	}
}


void parseARP (xmlDocPtr doc, xmlNodePtr cur) {
	cur = cur->xmlChildrenNode;
	apsr_output("- ARP Header -\n");
	
	while(cur != NULL) {
		if((!xmlStrcmp(cur->name, (const xmlChar *) "hardware_type"))) {
			printf("  Hardware Type: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "protocol_type"))) {
			printf("  Protocol Type: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "hardware_length"))) {
			printf("  Hardware Len: %s, ",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "protocol_length"))) {
			printf("Protocol Len: %s, ",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if((!xmlStrcmp(cur->name, (const xmlChar *) "opcode"))) {
			printf("Opcode: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if(!xmlStrcmp(cur->name, (const xmlChar *) "sender_hw_address")) {
			printf("  Sender Hardware Addr: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if(!xmlStrcmp(cur->name, (const xmlChar *) "sender_pr_address")) {
			printf("  Sender Protocol Addr: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if(!xmlStrcmp(cur->name, (const xmlChar *) "target_hw_address")) {
			printf("  Target Hardware Addr: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		if(!xmlStrcmp(cur->name, (const xmlChar *) "target_pr_address")) {
			printf("  Target Protocol Addr: %s\n",
			  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		}

		cur=cur->next;
	}
}

