#include "../include/aprecv-general.h"

int print_arp_packet (struct pcap_pkthdr *header, unsigned char *packet, int opt_size) {
  static struct arp_pkt_header *arp_header;
  static unsigned short hardware_type, protocol_type, opcode;
  static unsigned int x, hardware_length, protocol_length;
  static unsigned char *sender_hw_addr, *sender_pr_addr, *target_hw_addr, *target_pr_addr;

  arp_header = (struct arp_pkt_header *)(packet+data_link_size+opt_size);
	
  hardware_type = apsr_ntohs(arp_header->arp_hat);
  protocol_type = apsr_ntohs(arp_header->arp_pat);
  opcode 	= apsr_ntohs(arp_header->arp_oc);
  
  apsr_output("<packet:arp>\n");

  if(expand_arp || expand_all) {  
    apsr_output("<packet:hardware_type>%d</packet:hardware_type>",(int) hardware_type);
  
    if(verbose) {
    	  apsr_output("<packet:hardware_type_verbose>");
	  
	  switch(hardware_type) {
		  case ARP_HWTYPE_E10:
		    apsr_output("Ethernet 10 MBit/FDDI");
		    break;
		  case ARP_HWTYPE_EE:
		    apsr_output("Experimental Ethernet 3 Mbit)");
 		    break;
		  case ARP_HWTYPE_AX25:
		    apsr_output("Amateur Radio AX.25");
		    break;
		  case ARP_HWTYPE_PTR:
		    apsr_output("Proteon ProNet Token Ring");
		    break;
		  case ARP_HWTYPE_CHS:
		    apsr_output("Chaos");
		    break;
		  case ARP_HWTYPE_IEEE:
		    apsr_output("IEEE 802.x networks");
		    break;
		  case ARP_HWTYPE_ARC:
		    apsr_output("ARCNet");
		    break;
		  case ARP_HWTYPE_HCH:
		    apsr_output("HyperChannel");
		    break;
		  case ARP_HWTYPE_LANS:
		    apsr_output("LanStar");
		    break;
		  case ARP_HWTYPE_ASA:
		    apsr_output("Autonet Short Address");
		    break;
		  case ARP_HWTYPE_LTL:
		    apsr_output("LocalTalk");
		    break;
		  case ARP_HWTYPE_LN:
		    apsr_output("LocalNet [IBM PCNET/SYSTEK]");
		    break;
		  case ARP_HWTYPE_UL:
		    apsr_output("Ultra Link");
		    break;
		  case ARP_HWTYPE_SMDS:
		    apsr_output("SMDS");
		    break;
		  case ARP_HWTYPE_FR:
		    apsr_output("FrameRelay");
		    break;
		  case ARP_HWTYPE_ATM1:
		    apsr_output("ATM");
		    break;
		  case ARP_HWTYPE_HDLC:
		    apsr_output("HDLC");
		    break;
		  case ARP_HWTYPE_FC:
		    apsr_output("FiberChannel");
		    break;
		  case ARP_HWTYPE_ATM2:
		    apsr_output("ATM");
		    break;
		  case ARP_HWTYPE_SLIP: 
		    apsr_output("Serial Line");
		    break;
		  case ARP_HWTYPE_ATM3:
		    apsr_output("ATM");
		    break;
		  default:
		    apsr_output("unknown");
		    break;
	  }

	  apsr_output("</packet:hardware_type_verbose>\n");
    }  
		  
    apsr_output("<packet:protocol_type>%d</packet:protocol_type>\n",(int) protocol_type);

    if(verbose) {
    	apsr_output("<packet:protocol_type_verbose>");
	  switch(protocol_type) {
		  case ARP_PROTOTYPE_IP:
		    apsr_output("IPv4");	// IPv4 
		    break;
		  default:
		    apsr_output("unknown");
		    break;
	  }
	  
	apsr_output("</packet:protocol_type_verbose>\n");
    }   
  
    apsr_output("<packet:hardware_length>%d</packet:hardware_length>\n", arp_header->arp_hal);
    apsr_output("<packet:protocol_length>%d</packet:protocol_length>\n", arp_header->arp_pal);
    apsr_output("<packet:opcode>%d</packet:opcode>\n", (int) opcode);

    if(verbose) {
    	  apsr_output("<packet:opcode_verbose>");
	  
	  switch(opcode) {
		  case ARP_REQUEST:
		    apsr_output("ARP request");
		    break;
		  case ARP_REPLY:
		    apsr_output("ARP reply");
		    break;
		  case ARP_RREQUEST:
		    apsr_output("RARP request");
		    break;
		  case ARP_RREPLY:
		    apsr_output("RARP reply");
		    break;
                  case ARP_DRREQUEST:
                    apsr_output("DRARP request");
                    break;
                  case ARP_DRREPLY:
                    apsr_output("DRARP reply");
                    break;
                  case ARP_DRERROR:
                    apsr_output("DRARP error");
                    break;
		  case ARP_InREQUEST:
		    apsr_output("InARP request");
		    break;
		  case ARP_InREPLY: 
		    apsr_output("InARP reply");
		    break;
		  case ARP_NAK:
		    apsr_output("ATM ARP NAK");
		    break;
		  case ARP_MARSREQUEST:
		    apsr_output("MARS-Request");
		    break;
                  case ARP_MARSMULTI:
                    apsr_output("MARS-Multi");
                    break;
                  case ARP_MARSMSERV:
                    apsr_output("MARS-MServ");
                    break;
                  case ARP_MARSJOIN:
                    apsr_output("MARS-Join");
                    break;
                  case ARP_MARSLEAVE:
                    apsr_output("MARS-Leave");
                    break;
                  case ARP_MARSNAK:
                    apsr_output("MARS-Nak");
                    break;
                  case ARP_MARSUNSERV:
                    apsr_output("MARS-UnServ");
                    break;
                  case ARP_MARSSJOIN:
                    apsr_output("MARS-SJoin");
                    break;
                  case ARP_MARSSLEAVE:
                    apsr_output("MARS-SLeave");
                    break;
                  case ARP_MARSGRREQUE:
                    apsr_output("MARS-Grouplist-Request");
                    break;
                  case ARP_MARSGRREPLY:
                    apsr_output("MARS-Grouplist-Reply");
                    break;
                  case ARP_MARSREDMAP:
                    apsr_output("MARS-Redirect-Map");
                    break;
                  case ARP_MAPOSUNARP:
                    apsr_output("MAPOS-UNARP");
                    break;
		  default:
		    apsr_output("unknown");
		    break;
	  }

	  apsr_output("</packet:opcode_verbose>");
    }
  
    if(arp_header->arp_hal > 0)
      hardware_length = arp_header->arp_hal - 1;
    else 
      hardware_length = arp_header->arp_hal;
    if(arp_header->arp_pal > 0)
      protocol_length = arp_header->arp_pal - 1;
    else 
      protocol_length = arp_header->arp_pal;
    apsr_output("<packet:sender_hw_address>");
    if(hardware_length != 0) {
      sender_hw_addr = xmalloc(arp_header->arp_hal);
      strncpy(sender_hw_addr, (packet+data_link_size+ARP_HDR_LEN), arp_header->arp_hal);
      for(x = 0; x <= hardware_length; x++) {
 	if(x == hardware_length) { 
	  apsr_output("%2.2x", sender_hw_addr[x]);
	  if(verbose) 
	    resolve_mac(sender_hw_addr);
       	}
       	else 
          apsr_output("%2.2x:", sender_hw_addr[x]);
      }
      free(sender_hw_addr);
    }
    else {
      apsr_output("(None given, because hardware length is zero)");
    }
    
    apsr_output("</packet:sender_hw_address>\n");

    
    apsr_output("<packet:sender_pr_address>");
    if(protocol_type == APSR_ETHERTYPE_IP && arp_header->arp_pal == 4 && protocol_length != 0) {
      sender_pr_addr = xmalloc(arp_header->arp_pal);
      strncpy(sender_pr_addr, (packet+data_link_size+ARP_HDR_LEN+arp_header->arp_hal), arp_header->arp_pal);
      for(x = 0; x <= protocol_length; x++) {
        if(x == protocol_length) {
	  apsr_output("%u", sender_pr_addr[x]);
//    	  if(verbose) 
//	    resolve_ip(sender_pr_addr);
       	}
        else 
          apsr_output("%u.", sender_pr_addr[x]);
      }
      free(sender_pr_addr);
    }
    else if(protocol_type != APSR_ETHERTYPE_IP && arp_header->arp_pal != 4) 
      apsr_output("<Not implemented @ the moment>");
    else {
      apsr_output("(None given, because protocol length is zero)"); 
    }
    apsr_output("</packet:sender_pr_address>\n");
    
    apsr_output("<packet:target_hw_address>");
    if(hardware_length != 0) {
      target_hw_addr = xmalloc(arp_header->arp_hal);
      strncpy(target_hw_addr, (packet+data_link_size+ARP_HDR_LEN+arp_header->arp_hal+arp_header->arp_pal), arp_header->arp_hal);
    for(x = 0; x <= hardware_length; x++) {
	if(x == hardware_length) { 
	  apsr_output("%2.2x", target_hw_addr[x]);
	   if(verbose) 
	     resolve_mac(target_hw_addr);
	}
	else
	  apsr_output("%2.2x:", target_hw_addr[x]);
      }
      free(target_hw_addr);
    }
    else {
      apsr_output("(None given, because hardware length is zero)");
    }
    apsr_output("</packet:target_hw_address>\n");
    
    apsr_output("<packet:target_pr_address>");
    if(protocol_type == APSR_ETHERTYPE_IP && arp_header->arp_pal == 4 && protocol_length != 0) {
      target_pr_addr = xmalloc(arp_header->arp_pal);
      strncpy(target_pr_addr, (packet+data_link_size+ARP_HDR_LEN+arp_header->arp_hal+arp_header->arp_pal+arp_header->arp_hal), arp_header->arp_pal);
      for(x = 0; x <= protocol_length; x++) {
        if(x == protocol_length) {
	  apsr_output("%u", target_pr_addr[x]);
//  	  if(verbose) 
//	    resolve_ip(target_pr_addr);
 	}
        else 
          apsr_output("%u.", target_pr_addr[x]);
      }
      free(target_pr_addr);
    }
    else if(protocol_type != APSR_ETHERTYPE_IP && arp_header->arp_pal != 4)
      apsr_output("<Not implemented @ the moment>");
    else {
      apsr_output("(None given, because protocol length is zero)");
    }
    apsr_output("</packet:target_pr_address>\n");
    
    if(check_errors) 
	    print_arp_errors(header, packet);

  }
  
  if(statistic) 
        stats.c.arp++;

  apsr_output("</packet:arp>\n");
  apsr_output("</packet:packet>\n");

  return(RET_SUCCESS);
}
