#include "../include/aprecv-general.h"

int print_ethernet_packet (struct pcap_pkthdr *header, unsigned char *packet, int opt_size) {
  struct ethernet_pkt_header *ethernet_header;
  struct vlan_pkt_header *vlan_header=0;
  unsigned short ethernet_type=0, ethernet_snap_type=0, ethernet_vlan_type=0; 
  unsigned short next_protocol=0, vlan=0;

  ethernet_header = (struct ethernet_pkt_header *) packet;
  ethernet_type = apsr_ntohs(ethernet_header->type);

  apsr_output("<packet:ethernet>\n");

  if(header->caplen < APSR_ETH) {
	  apsr_output("\nError: Captured data length(%d bytes) < Ethernet packet header length.\n", (int) header->caplen);
	  if(statistic)
		  stats.tr.ethernetII++;
	  goto end_ethernet;
  }

  if(expand_ethernet || expand_all) {
    apsr_output("<packet:destination>%s</packet:destination>\n", ether2str(ethernet_header->destination));
    
    if(verbose)
         resolve_mac(ethernet_header->destination);
	 
    apsr_output("<packet:source>%s</packet:source>\n", ether2str(ethernet_header->source));
    
    if(verbose)
 	 resolve_mac(ethernet_header->source);
  }
    
  if(ethernet_type == APSR_ETHERTYPE_VLAN) {       /* VLAN 802.1p/1q frames */
        opt_size += 4;
        vlan_header = (struct vlan_pkt_header *)(packet+APSR_ETH);
        if(expand_ethernet || expand_all) {
          apsr_output("<packet:tpid>0x%04x</packet:tpid>\n", ethernet_type);
	  vlan = apsr_ntohs(vlan_header->vlan);
	  apsr_output("<packet:priority>%d</packet:priority><packet:tr_cfi>%d</packet:tr_cfi>\n", vlan >> 13, vlan & 0x1000);
	  apsr_output("<packet:vlan-id>%d</vlan-id>\n", vlan & 0x0fff);
        }
	
        ethernet_vlan_type = apsr_ntohs(vlan_header->type);
        if(ethernet_vlan_type <= 1500) {
          opt_size += 3;
          if(expand_ethernet || expand_all) {
	    apsr_output("<packet:length>%d</packet:lenght>\n",(int)ethernet_vlan_type);
            apsr_output("<packet:dsap>0x%02x</packet:dsap><packet:ssap>0x%02x</packet:ssap>\n", 
                      vlan_header->ext.snap.dsap,
                      vlan_header->ext.snap.ssap);
            apsr_output("<packet:control>0x%02x</packet:control>\n",
                      vlan_header->ext.snap.control);
          } 
           
          if(vlan_header->ext.snap.dsap == 0xAA &&
             vlan_header->ext.snap.ssap == 0xAA &&
             vlan_header->ext.snap.control == 0x03) {                 
            opt_size += 5;
            if(expand_ethernet || expand_all) {
              apsr_output("<packet:oui>0x%02x 0x%02x 0x%02x</packet:oui>\n",
                            vlan_header->ext.snap.org_id[0],
                            vlan_header->ext.snap.org_id[1],
                            vlan_header->ext.snap.org_id[2]);

	      if(verbose)
                print_snap_oui(vlan_header->ext.snap.org_id);
	    }
 
            ethernet_snap_type = apsr_ntohs(vlan_header->ext.snap.type);
            
            if(expand_ethernet || expand_all) {
              apsr_output("<packet:type>0x%04x</packet:type>\n", ethernet_snap_type);
              
              if(verbose) {
		      if((APSR_24TO32(ethernet_header->ext.snap.org_id) ==
				APSR_SNAP_OUI_CISCO) && 
	              		(ethernet_snap_type == APSR_SNAP_PID_CDP))
	                      apsr_output("<packet:oui_verbose> (CDP)</packet:oui_verbose>\n");
                      else
			      apsr_output("<packet:oui_verbose> (unknown)</packet:oui_verbose>\n");
	      }
	    }
       
            if(statistic) 
              stats.c.ethernet8023snap++;

            if((APSR_24TO32(ethernet_header->ext.snap.org_id) ==
                            APSR_SNAP_OUI_CISCO) &&
	                    (ethernet_snap_type == APSR_SNAP_PID_CDP))
		    print_cdp_packet(header, packet, opt_size, ethernet_vlan_type-opt_size);
	  }
          else {
            if(statistic)
              stats.c.ethernet8023++;
          }
        }
        else {
          if(expand_ethernet || expand_all) {  
   	    apsr_output("<packet:vlan_type>0x%04x</packet:vlan_type>\n", ethernet_vlan_type);
	    if(verbose) 
		  print_ethernet_proto(ethernet_vlan_type);
	  }
          next_protocol = ethernet_vlan_type;
        }
	
	if(statistic)
		stats.c.vlan++;
  }
  else if(ethernet_type > 1500) {	/* Ethernet II or 802.x/SNAP frames */
        next_protocol = ethernet_type;
        if(expand_ethernet || expand_all) {
  	  apsr_output("<packet:typeII>0x%04x</packet:typeII>\n", ethernet_type);
  
          if(verbose) 
		print_ethernet_proto(ethernet_type);
	}  
	  
        if(statistic)
		stats.c.ethernetII++;  
  }
	
  else if (ethernet_type <= 1500) {	/* 802.3, for now it a length field */
        if(ethernet_header->ext.snap.dsap == 0xff && ethernet_header->ext.snap.ssap == 0xff) {
                opt_size += 2;
                if(expand_ethernet || expand_all) 
		  apsr_output("<packet:length_802.3raw>%d</packet:length_802.3raw>\n",(int) ethernet_type);

                if(statistic)
                  stats.c.ethernet8023raw++;

                print_ipx_packet(header, packet, opt_size);
	}
	else {
	        opt_size += 3;
                if(expand_ethernet || expand_all) {
			  apsr_output("<packet:length_802.3>%d</packet:length_802.3>\n",(int) ethernet_type);
         		  apsr_output("<packet:dsap>0x%02x</packet:dsap><packet:ssap>0x%02x</packet:ssap>\n",
				  ethernet_header->ext.snap.dsap, 
				  ethernet_header->ext.snap.ssap);
		          apsr_output("<packet:control>0x%02x</packet:control>\n",
				  ethernet_header->ext.snap.control);
	        }
		  
                if(ethernet_header->ext.snap.dsap == 0xAA &&
                   ethernet_header->ext.snap.ssap == 0xAA &&
                   ethernet_header->ext.snap.control == 0x03) {
                  opt_size += 5;
                  if(expand_ethernet || expand_all) {
     		    apsr_output("<packet:oui>0x%02x 0x%02x 0x%02x</packet:oui>\n",
  				  ethernet_header->ext.snap.org_id[0], 
				  ethernet_header->ext.snap.org_id[1],
				  ethernet_header->ext.snap.org_id[2]);

		    if(verbose) 
		      print_snap_oui(ethernet_header->ext.snap.org_id);
		  }
                 
                  ethernet_snap_type = apsr_ntohs(ethernet_header->ext.snap.type);
 
                  if(expand_ethernet || expand_all) {
	            apsr_output("<packet:snap_type>0x%04x</packet:snap_type>\n", ethernet_snap_type);
                 
                    if(verbose) {
			    if(APSR_24TO32(ethernet_header->ext.snap.org_id) == APSR_SNAP_OUI_CISCO && ethernet_snap_type == APSR_SNAP_PID_CDP) 
				    apsr_output("<packet:oui_verbose> (CDP)</packet:oui_verbose>\n");
			    else 
				    apsr_output("<packet:oui_verbose> (unknown)</packet>\n");
                    }				   
	          }

                  if(statistic)
		    stats.c.ethernet8023snap++;

		  if((APSR_24TO32(ethernet_header->ext.snap.org_id) == 
				  APSR_SNAP_OUI_CISCO) && 
				  (ethernet_snap_type == APSR_SNAP_PID_CDP)) 
		          print_cdp_packet(header, packet, opt_size, ethernet_type-opt_size);
                }
                else {
                    if(statistic)
                      stats.c.ethernet8023++;
                }
        }
  }

  apsr_output("</packet:ethernet>\n\n");

  if(next_protocol != 0) 
	  switchto_ethernet_proto(header, packet, opt_size, next_protocol);

end_ethernet:

  return(RET_SUCCESS);
}

void switchto_ethernet_proto(struct pcap_pkthdr *header, unsigned char *packet, int opt_size, unsigned short protocol) {
  switch(protocol) {
	  case APSR_ETHERTYPE_ARP:
	    print_arp_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_REVARP:
	    print_arp_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_IP:
	    print_ipv4_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_IP6:
	    print_ipv6_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_PPP:
	    print_ppp_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_PPPOE_D:
	    print_pppoe_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_PPPOE_S:
	    print_pppoe_packet(header, packet, opt_size);
	    break;
          case APSR_ETHERTYPE_IPX:
            print_ipx_packet(header, packet, opt_size);
            break;
          default:
            apsr_output("\n");
            break;
  }
}

void print_ethernet_proto(unsigned short protocol) {
  apsr_output("<packet:protocol>");
  switch(protocol) {
	  case APSR_ETHERTYPE_ARP:
	    apsr_output("(ARP)");
	    break;
	  case APSR_ETHERTYPE_REVARP:
	    apsr_output("(Reverse ARP)");
	    break;
	  case APSR_ETHERTYPE_IP:
	    apsr_output("(IPv4)");
	    break;
	  case APSR_ETHERTYPE_IP6:
	    apsr_output("(IPv6)");
	    break;
	  case APSR_ETHERTYPE_PPP:
	    apsr_output("(PPP)");
	    break;
	  case APSR_ETHERTYPE_LOOPBACK:
	    apsr_output("(Ethernet Loopback)");
	    break;
	  case APSR_ETHERTYPE_PPPOE_D:
	    apsr_output("(PPPoE - Discovery Stage)");
	    break;
	  case APSR_ETHERTYPE_PPPOE_S:
	    apsr_output("(PPPoE - Session Stage)");
	    break;
          case APSR_ETHERTYPE_IPX:
            apsr_output("(IPX)");
            break;
          default:
            apsr_output("(unknown type)");
            break;
  }
  
  apsr_output("</packet:protocol>");
}

void print_snap_oui(unsigned char *oui) {
  static unsigned int orgcode;

  orgcode = APSR_24TO32(oui);

  apsr_output("<packet:snap_oui>");
  
  switch(orgcode) {
	  case APSR_SNAP_OUI_ENCETH:
            apsr_output("(Encapsulated ethernet)");
            break;
	  case APSR_SNAP_OUI_CISCO:
	    apsr_output("(Cisco protocols)");
	    break;
          case APSR_SNAP_OUI_CISCO_90:
	    apsr_output("(Cisco bridging)");
	    break;
	  case APSR_SNAP_OUI_BRIETH:
	    apsr_output("(Bridged ethernet)");
	    break;
	  case APSR_SNAP_OUI_APPLETALK:
	    apsr_output("(Appletalk)");
	    break;
	  default:
	    apsr_output("(unkown)");
	    break;
  }

  apsr_output("</packet:snap_oui>\n");
}

