#include "../include/aprecv-general.h"

int (*aprecv_print_index[])(struct pcap_pkthdr*, unsigned char *, int) = { print_ethernet_packet, print_ppp_packet, print_pppoe_packet, print_sll_packet, print_802_11_packet, print_prism_mon_packet, print_null_packet, print_slip_packet, print_token_packet };

int start_sniffing(char *arg, int promisc, int snaplen, char *rule, int opt_filter, int max_packets) {
  static char error_buffer[PCAP_ERRBUF_SIZE];
  static struct bpf_program filter;
  struct in_addr localnet, netmask;
  
  #ifdef APRECV_DEBUG
  static int now_snaplen;
  #endif

  if (device == NULL && pcap_log_file == NULL && (device = pcap_lookupdev(error_buffer)) == NULL) {
  	  #ifdef __OSTYPE_QNX
	  ERROR_EXIT("Live packet capturing does not work under the QNX operating system!");
	  #endif
		
	  apsr_stream_output((int) stderr, "Error: Can't open device %s, because: %s\n\n", 
	    device, error_buffer);
	  exit(EXIT_FAILURE);
  } 
  else if(pcap_log_file) {
	  if((recv_d = pcap_open_offline(pcap_log_file, error_buffer)) == NULL) {
	  	apsr_stream_output((int) stderr, "Error: Can't open pcap file %s, because: %s\n\n",
		  pcap_log_file, error_buffer);	  
		exit(EXIT_FAILURE);
	  }
	  else {
		  apsr_output("Read packet(s) from file: %s\n", pcap_log_file);
		  if(verbose) {
			  if(pcap_is_swapped(recv_d) == 1) 
				  apsr_output("Info: file use different byte order, than local system.\n");
		  }
	  }
  }
  else {
	  apsr_output("Receive packet(s) on device: %s\n", device); 
  }
	
  if(pcap_log_file == NULL && (recv_d = pcap_open_live(device, snaplen, promisc, 1000, error_buffer)) == NULL) {
	  apsr_stream_output((int) stderr, "Error: Can't open device %s, because: %s\n\n", device, error_buffer);
	  exit(EXIT_FAILURE);
  }

  #ifdef APRECV_DEBUG
  now_snaplen = pcap_snapshot(recv_d);

  if(snaplen < now_snaplen) {
	  apsr_stream_output((int) stderr, "Error: Snaplen changed by pcap_open_live to %d. Not important.\n", now_snaplen);
	  // snaplen = now_snaplen;	/* Not needed yet */
  }
  #endif
  
  if(pcap_lookupnet(device, &localnet.s_addr, &netmask.s_addr, error_buffer) < 0) {
	  localnet.s_addr = 0;
	  netmask.s_addr  = 0;
	  
	  #ifdef APRECV_DEBUG
	  /* Failure can happen, some devices/protocols are not really implemented */
	  apsr_stream_output((int) stderr, "Error: Can't determine localnet/netmask, because: %s. Not important.\n", error_buffer);
	  #endif
  }
  else if(verbose) {
	  apsr_output("Looking at: %s/", inet_ntoa(localnet));
	  apsr_output("%s\n", inet_ntoa(netmask));
  }
			  
  if(rule) {
	  if(pcap_compile(recv_d, &filter, rule, opt_filter, netmask.s_addr) < 0) {
		  apsr_stream_output((int) stderr, "Error: Can't compile(pcap_compile) filter rule, because: %s\n", pcap_geterr(recv_d));
		  exit(EXIT_FAILURE);
	  }
	  if(pcap_setfilter(recv_d, &filter) < 0) {
            	  apsr_stream_output((int) stderr, "Error: Can't set(pcap_setfilter) filter rule, because: %s\n", pcap_geterr(recv_d));
		  exit(EXIT_FAILURE);
	  }
	  apsr_output("Filter rule: %s\n", rule);
  } 
  
  if(aprecv_log_raw_file) {
	  if((dump = pcap_dump_open(recv_d, aprecv_log_raw_file)) == NULL) {
		  apsr_stream_output((int) stderr, "Error: Can't open(pcap_dump_open) logfile, because: %s\n", pcap_geterr(recv_d));
	   	  exit(EXIT_FAILURE);
	  }
  }
  
  if((link_layer = pcap_datalink(recv_d)) < 0) {
	  apsr_stream_output((int) stderr, "Error: Can't get datalink information: %s\n", pcap_geterr(recv_d));
          exit(EXIT_FAILURE);
  }		    
  
  switch (link_layer) {
	  case APSR_DLT_NULL:
	  #ifdef __OSTYPE_OPENBSD
	  case APSR_DLT_LOOP:			/* OpenBSD only */
	  #endif
      	    data_link_size = 4;
	    index_number = 6;
            break;
          case APSR_DLT_PPP:			/* PPP */
	    data_link_size = 4;
	    index_number = 1;
	    break;
	  case APSR_DLT_ATM_RFC1483:
	    data_link_size = 0;			/* Need 8 ??? */
	    break;
          case APSR_DLT_EN10MB:			/* Ethernet II */
          case APSR_DLT_EN3MB:			/* Experimental Ethernet */
	    data_link_size = 14;
	    index_number = 0;
	    break;
	  case APSR_DLT_IEEE802:		/* TokenRing(802.5) */
            data_link_size = 14;
            index_number = 8;
            break;
	  #ifdef APSR_DLT_PPP_ETHER
	  case APSR_DLT_PPP_ETHER:		/* PPPoE / UNTESTED */
	    data_link_size = 14;
	    index_number = 2;
	    break;
	  #endif
          #ifdef APSR_DLT_IEEE802_11
	  case APSR_DLT_IEEE802_11:			
	    data_link_size = 0;			/* No need */
	    index_number = 4;
	    break; 
          #endif
	  #ifdef APSR_DLT_PRISM_HEADER		/* Since Libpcap 0.7.1 */
	  case APSR_DLT_PRISM_HEADER:	
	    data_link_size = 144;		
	    index_number = 5;
	    break;
          #endif
          case APSR_DLT_PPP_BSDOS:			
	    data_link_size = 24;
 	    break;
          case APSR_DLT_SLIP:
 	    data_link_size = 16;		/* Slip */
	    index_number=7;
	    break;
	  case APSR_DLT_SLIP_BSDOS:
	    #if __OSTYPE_OPENBSD
            data_link_size = 16;		/* Free/Open/Net/BSD/+i only */
	    #else
	    data_link_size = 24;
	    #endif
	    break;
          case APSR_DLT_FDDI:
	    data_link_size = 21;		/* FDDI */
	    break;
          #ifdef APSR_DLT_RAW			/* Not supported in older 
	 					   libpcap versions and archs */
	  case APSR_DLT_RAW:
	    data_link_size = 0;
	    index_number = 6;
	    break;
	  #endif
	  #ifdef APSR_DLT_LINUX_SLL		/* Not suppport in older
						   libpcap versions */
	  case APSR_DLT_LINUX_SLL:
	    data_link_size = 0;		
	    index_number = 3;
	    break;
	  #endif 
	  default:
	    apsr_stream_output((int) stderr, "Error: Unknown device/datalink type");
	    #ifdef APRECV_DEBUG
	    apsr_output("(%d)", link_layer);
 	    #endif
	    apsr_output(".\n\n");
	    exit(EXIT_FAILURE);
  }
  #ifdef APRECV_DEBUG
  apsr_output("Use datalink type %d.\n", link_layer);
  #endif
 
  apsr_output("<?xml version=\"1.0\"?>\n");
  apsr_output("<packet:packet type:packet=\"http://www.aa-security.de\">\n");

  if(pcap_loop(recv_d, max_packets, (pcap_handler)recv_packet, (unsigned char *) dump) < 0) {
	  #ifdef APRECV_DEBUG
	  apsr_stream_output((int) stderr, "Error while starting pcap_loop: %s\n", pcap_geterr(recv_d));
          #endif
	  apsr_output("</packet:packet>\n");
	  apsr_output("</packet:packet>\n");
	  pcap_close(recv_d);
	  exit(EXIT_FAILURE);
  }

  return(RET_SUCCESS);
}

int recv_packet(unsigned char *user, struct pcap_pkthdr *header, unsigned char *packet) {
  static int sec;
  static unsigned long diff_receive_time;
  static float utilization;
  time_t now_receive_time;

  stats.c.all++;

  if(beep_on)
	  apsr_output("\a");

  all_counted_bytes += header->len;

  /* print out the time when the packet comes in & utilization */
  
  sec = header->ts.tv_sec % 86400;
//  apsr_output("\n%02d:%02d:%02d.%06u - Received packet(%d bytes length)...", sec / 3600, (sec % 3600) / 60, sec % 60, (unsigned)header->ts.tv_usec, header->len);
  apsr_output("<packet:packet>\n");
  apsr_output("\n<packet:info>\n");
  apsr_output("<packet:number>%llu</packet:number>\n", stats.c.all);
  apsr_output("<packet:arrival>%02d:%02d:%02d.%06u</packet:arrival>\n", sec / 3600, (sec % 3600) / 60, sec % 60, (unsigned)header->ts.tv_usec);
  apsr_output("<packet:length>%d</packet:length>\n", header->len);
  apsr_output("</packet:info>\n");
 
  if(statistic) {
	  now_receive_time = time(NULL);
	  diff_receive_time = difftime(now_receive_time, start_receive_time);
	  if(diff_receive_time > 0) {	/* Needed, overwhise core dump */
		  utilization = all_counted_bytes / diff_receive_time;
		  apsr_output(" Utilization: %.0f B/s", utilization);
	  }
	  else {
		  apsr_output(" Utilization: n.a.");
          }
  }
  apsr_output("\n");
 
  (*aprecv_print_index[index_number])(header, packet, 0);
  
  if(print_raw_hex && !print_payload_ip_hex && !print_payload_pppoe_hex && 
     !print_payload_tcp_text && !print_payload_udp_text && 
     !print_payload_tcp_hex && !print_payload_udp_hex) {
	 print_raw_in_hex(packet, header->len-1);
  }
	 
  if(aprecv_log_raw_file) 
         pcap_dump(user, header, packet);
  
  if(fflush(stdout) == EOF) {
 	 #ifdef APRECV_DEBUG
	 apsr_stream_output((int) stderr, "Error: fflush(stdout) %s.\n", strerror(errno));
	 #endif
  }
 
  return(RET_SUCCESS);  
}
