/* Graphical User Interface for aprecv
 --------------------------------------
Options of aprecv:
Usage: aprecv [Options]
 -d    --device <device>        Device to open
 -f    --filter "filter rule"   Set bpf filter rule, optimized
 -D    --daemon                 Run as daemon
 -p    --promisc <0|1>          Promisc mode off(0)/on(1), default on
 -c    --count                  Count all bytes received
       --statistic              Count all packets received
 -s    --snaplen <up to 65535>  Set the snaplen for pcap
       --max-packets <num>      Maximum packets to receive, default is loop
       --check-for-errors       Check for errors in packets an print them
       --calc-checksums         Calculate checksums, print correct if wrong
       --expand-<protocol>      Expand the <protocol> header
 -v    --verbose                Print out more information
 -l    --logfile <file>         File to log the packets
 -r    --logfile-raw <file>     File to log the packets(raw)
 -R    --print-raw-hex          Print raw packet in hex
       --print-<protocol>-hex   Print payload of <protocol> in hex
       --print-<protocol>-text  Print payload of <protocol> in text
       --module <file>          Load shared module <file>
       --module-raw             Give the whole packet to the module
       --module-ignore          Ignore the builtin BPF filter of the module
       --server-addr <ip>       Open connection to APSR server
       --server-port <port>     Connect to <port>, default 2777
 -h    --help                   Display this help message
 -V    --version                Display the version
*/

// gtk_widget_set_sensitive(GTK_WIDGET( ),0);



#include "../include/apsr-gui.h"
#include <apsrlib/apsrlib.h>


int check_options(void);


struct options {
  unsigned char *device;
  unsigned char *filter;
  unsigned char daemon1;
  unsigned char promisc;
  unsigned char count;
  unsigned char statistic;
  unsigned short int snaplen;
  unsigned short int maxpackets;
  unsigned char errorcheck;
  unsigned char checksum;
  unsigned char *expand;
  unsigned char verbose;
  unsigned char *logfile;
  unsigned char *logfile_raw;
  unsigned char print_raw_hex;
  unsigned char *print_proto_hex;
  unsigned char *print_proto_text;
  unsigned char *module;
  unsigned char module_raw;
  unsigned char module_ignore;
  unsigned char *serveraddr;
  unsigned char *serverport;
  unsigned char version;
}opt;

pid_t pid;


int check_options(void){
  FILE *pipe;
  char exec_buf[1024];
  char temp[1024];
  char buffer;
  char c;
  exec_buf[0]='\0';

  sprintf(temp, "aprecv ");
  strcat(exec_buf, temp);  
  if(opt.device != NULL) {
     sprintf(temp, "-d %s ", opt.device);
     strcat(exec_buf, temp);
  }
  if(opt.filter != NULL){
    sprintf(temp, "-f \"%s\" ", opt.filter);
    strcat(exec_buf, temp);
  }
  if(opt.daemon1 != 0){
    sprintf(temp, "-D ");
    strcat(exec_buf, temp);
  }
  if(opt.promisc != 0){
    sprintf(temp, "-p %u ", opt.promisc);
    strcat(exec_buf, temp);
  }
  if(opt.count != 0){
    sprintf(temp, "-c ");
    strcat(exec_buf, temp);
  }
  if(opt.statistic != 0){
    sprintf(temp, "--statistic ");
    strcat(exec_buf, temp);
  }
  if(opt.snaplen != 0){
    sprintf(temp, "-s %u ", opt.snaplen);
    strcat(exec_buf, temp);
  }
  if(opt.maxpackets !=0){
    sprintf(temp, "--max-packets %u ", opt.maxpackets);
    strcat(exec_buf, temp);
  }
  if(opt.errorcheck != 0){
    sprintf(temp, "--check-for-errors ");
    strcat(exec_buf, temp);
  }
  if(opt.checksum != 0){
    sprintf(temp, "--calc-checksums ");
    strcat(exec_buf, temp);
  }
  if(opt.expand != NULL){
    sprintf(temp, "--expand-%s ", opt.expand);
    strcat(exec_buf, temp);
  }
  if(opt.verbose != 0){
    sprintf(temp, "--verbose ");
    strcat(exec_buf, temp);
  }
  if(opt.logfile != NULL){
    sprintf(temp, "-l %s ",opt.logfile);
    strcat(exec_buf, temp);
  }
  if(opt.logfile_raw != NULL){
    sprintf(temp, "-r %s ",opt.logfile_raw);
    strcat(exec_buf,temp);
  }
  if(opt.print_raw_hex != 0){
    sprintf(temp, "-R ");
    strcat(exec_buf, temp);
  }
  if(opt.print_proto_hex != NULL){
    sprintf(temp, "--print-%s-hex ", opt.print_proto_hex);
    strcat(exec_buf, temp);
  }
  if(opt.print_proto_text != NULL){
    sprintf(temp, "--print-%s-text ", opt.print_proto_text);
    strcat(exec_buf, temp);
  }
  if(opt.module != NULL){
    sprintf(temp, "--module %s ", opt.module);
    strcat(exec_buf, temp);
  }
  if(opt.module_raw != 0){
    sprintf(temp, "--module-raw ");
    strcat(exec_buf, temp);
  }
  if(opt.module_ignore != 0){
    sprintf(temp, "--module-ignore ");
    strcat(exec_buf, temp);
  }
  if(opt.serveraddr != NULL){
    sprintf(temp, "--server-addr %s ", opt.serveraddr);
    strcat(exec_buf, temp);
  }
  if(opt.serverport !=NULL){
    sprintf(temp, "--server-port %s ", opt.serverport);
    strcat(exec_buf, temp);
  }
  if(opt.version != 0){
    pipe=popen("aprecv --version", "r");
    while((c=getc(pipe)) != EOF)
	    g_print("%c",c);
    pclose(pipe);
    g_print("\n");
    return(RET_SUCCESS);
  }

  //sprintf(temp, "\n");
  //strcat(exec_buf, temp);

  if((pid=fork()) == 0){
    pipe=popen(exec_buf, "r");  
    while((buffer=getc(pipe)) != EOF)
	  g_print("%c",buffer);
	  
	  pclose(pipe);  
  }
  return(RET_SUCCESS);
}

gint clicked_ok(GtkWidget *window) {

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(device))) {
    if(strlen(gtk_entry_get_text(GTK_ENTRY(device_txt))) != 0)  
      opt.device=g_strdup(gtk_entry_get_text(GTK_ENTRY(device_txt)));
    else
      opt.device=NULL;
  }
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(filter))) {
    if(strlen(gtk_entry_get_text(GTK_ENTRY(filter_txt))) != 0)
      opt.filter=g_strdup(gtk_entry_get_text(GTK_ENTRY(filter_txt)));
    else
      opt.filter=NULL;
  }
    
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(daemon1)))
    opt.daemon1=1;
  else
    opt.daemon1=0;
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(promisc)))
    opt.promisc=atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(promisc_box)->entry)));
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(count)))
    opt.count=1;
  else
    opt.count=0;
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(statistic)))
    opt.statistic=1;
  else
    opt.statistic=0;
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snaplen))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(snaplen_txt))) != 0)
      opt.snaplen=atoi(gtk_entry_get_text(GTK_ENTRY(snaplen_txt)));
    else
      opt.snaplen=0;
  }
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(max_packets))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(max_packets_txt))) != 0)
      opt.maxpackets=atoi(gtk_entry_get_text(GTK_ENTRY(max_packets_txt)));
    else
      opt.maxpackets=0;
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_for_errors)))
    opt.errorcheck=1;
  else
    opt.errorcheck=0;
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(calc_checksums)))
    opt.checksum=1;
  else
    opt.checksum=0;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(expand))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(expand_txt))) != 0)
      opt.expand=g_strdup(gtk_entry_get_text(GTK_ENTRY(expand_txt)));
    else
      opt.expand=NULL;
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(verbose)))
    opt.verbose=1;
  else
    opt.verbose=0;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(logfile))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(logfile_txt))) != 0)
       opt.logfile=g_strdup(gtk_entry_get_text(GTK_ENTRY(logfile_txt)));
    else
      opt.logfile=NULL;
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(logfile_raw))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(logfile_raw_txt))) != 0)
      opt.logfile_raw=g_strdup(gtk_entry_get_text(GTK_ENTRY(logfile_raw_txt)));
    else
      opt.logfile_raw=NULL; 
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(print_raw_hex)))
    opt.print_raw_hex=1;
  else
    opt.print_raw_hex=0;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(print_proto_hex))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(print_proto_hex_txt))) != 0)
      opt.print_proto_hex=g_strdup(gtk_entry_get_text(GTK_ENTRY(print_proto_hex_txt)));
    else
      opt.print_proto_hex=NULL;  
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(print_proto_text))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(print_proto_text_txt))) != 0)
      opt.print_proto_text=g_strdup(gtk_entry_get_text(GTK_ENTRY(print_proto_text_txt)));
    else
      opt.print_proto_text=NULL;
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(module))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(module_txt))) != 0)
      opt.module=g_strdup(gtk_entry_get_text(GTK_ENTRY(module_txt)));
    else
      opt.module=NULL;
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(module_raw)))
    opt.module_raw=1;
  else
    opt.module_raw=0;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(module_ignore)))
    opt.module_ignore=1;
  else
    opt.module_ignore=0;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(server_addr))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(server_addr_txt))) != 0)
      opt.serveraddr=g_strdup(gtk_entry_get_text(GTK_ENTRY(server_addr_txt)));
    else
      opt.serveraddr=NULL;  
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(server_port))){
    if(strlen(gtk_entry_get_text(GTK_ENTRY(server_port_txt))) != 0)
      opt.serverport=g_strdup(gtk_entry_get_text(GTK_ENTRY(server_port_txt)));
    else
      opt.serverport=NULL;  
  }

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(aprecv_version)))
    opt.version=1;
  else
    opt.version=0;

  check_options();  
  return(RET_SUCCESS);
}


void exit_aprecv(GtkWidget *window) {  
  kill(pid, 9);
  gtk_main_quit();
}

int main(int argc, char **argv) {
  /* Initalize Gtk spezified variables */
  GtkWidget *window; //, *scrollwindow;
  GtkWidget *table1; //, *button_table;
  GtkWidget *button_ok, *button_no;
  
//GtkObject *hadj;

  /* initialisize gtk */
  gtk_init(&argc, &argv);
  
  /* create main window */
  window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_default_size(GTK_WINDOW(window), 290, 585);


  /* create other stuff*/
  table1=gtk_table_new(25,3,FALSE);
  button_ok=gtk_button_new_with_label("OK");
  button_no=gtk_button_new_with_label("Cancel");

  device_txt=gtk_entry_new();
  filter_txt=gtk_entry_new();
  snaplen_txt=gtk_entry_new();
  max_packets_txt=gtk_entry_new();
  expand_txt=gtk_entry_new();
  logfile_txt=gtk_entry_new();
  logfile_raw_txt=gtk_entry_new();
  print_proto_hex_txt=gtk_entry_new();
  print_proto_text_txt=gtk_entry_new();
  module_txt=gtk_entry_new();
  server_addr_txt=gtk_entry_new();
  server_port_txt=gtk_entry_new();

  promisc_box=gtk_combo_new();
  promisc_data=g_list_append(promisc_data, "0");
  promisc_data=g_list_append(promisc_data, "1");
  gtk_combo_set_popdown_strings(GTK_COMBO(promisc_box), promisc_data);
  gtk_combo_set_use_arrows(GTK_COMBO(promisc_box), 1);

  /* create check buttons */
  device=gtk_check_button_new_with_label("Device to open");
  filter=gtk_check_button_new_with_label("Set bpf filter rule, optimized");
  daemon1=gtk_check_button_new_with_label("Run as deamon");
  promisc=gtk_check_button_new_with_label("Promisc mode on/off, default on");
  count=gtk_check_button_new_with_label("Count all bytes received");
  statistic=gtk_check_button_new_with_label("Count all packets recieved");
  snaplen=gtk_check_button_new_with_label("Set the snaplen for pcap");
  max_packets=gtk_check_button_new_with_label("Maximus packets to receive, default is loop");
  check_for_errors=gtk_check_button_new_with_label("Check for errors in packets and print them");
  calc_checksums=gtk_check_button_new_with_label("Calculate checksums, print correct if wrong");
  expand=gtk_check_button_new_with_label("Expand the given protocol header");
  verbose=gtk_check_button_new_with_label("Print out more information");
  logfile=gtk_check_button_new_with_label("File to log the packets(hex)");
  logfile_raw=gtk_check_button_new_with_label("File to log the packets(raw)");
  print_raw_hex=gtk_check_button_new_with_label("Print raw packet in hex");
  print_proto_hex=gtk_check_button_new_with_label("Print payload of the given protocol in hex");
  print_proto_text=gtk_check_button_new_with_label("Print payload of the given protocol in text");
  module=gtk_check_button_new_with_label("Load the given shared module");
  module_raw=gtk_check_button_new_with_label("Give the whole packet to the module");
  module_ignore=gtk_check_button_new_with_label("Ignore the builtin BPF filter of the module");
  server_addr=gtk_check_button_new_with_label("Open connection to APSR server");
  server_port=gtk_check_button_new_with_label("Connect to the given port");
  aprecv_version=gtk_check_button_new_with_label("Display the version");


  /* packing check buttons into table */
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(device), 0,1,0,1);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(device_txt), 1,2,0,1);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(filter), 0,1,1,2);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(filter_txt), 1,2,1,2);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(daemon1), 0,1,2,3);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(promisc), 0,1,3,4);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(promisc_box), 1,2,3,4);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(count), 0,1,4,5);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(statistic), 0,1,5,6);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(snaplen), 0,1,6,7);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(snaplen_txt), 1,2,6,7);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(max_packets), 0,1,7,8);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(max_packets_txt), 1,2,7,8);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(check_for_errors), 0,1,8,9);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(calc_checksums), 0,1,9,10);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(expand), 0,1,10,11);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(expand_txt), 1,2,10,11);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(verbose), 0,1,11,12);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(logfile), 0,1,12,13);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(logfile_txt), 1,2,12,13);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(logfile_raw), 0,1,13,14);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(logfile_raw_txt), 1,2,13,14);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(print_raw_hex), 0,1,14,15);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(print_proto_hex), 0,1,15,16);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(print_proto_hex_txt), 1,2,15,16);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(print_proto_text), 0,1,16,17);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(print_proto_text_txt), 1,2,16,17);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(module), 0,1,17,18);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(module_txt), 1,2,17,18);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(module_raw), 0,1,18,19);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(module_ignore), 0,1,19,20);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(server_addr), 0,1,20,21);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(server_addr_txt), 1,2,20,21);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(server_port), 0,1,21,22);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(server_port_txt), 1,2,21,22);

  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(aprecv_version), 0,1,22,23);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(button_ok), 0,1,24,25);
  gtk_table_attach_defaults(GTK_TABLE(table1), GTK_WIDGET(button_no), 1,2,24,25);

  /* Set Signal Handlers */
  gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
  gtk_signal_connect_object(GTK_OBJECT(button_no), "clicked", GTK_SIGNAL_FUNC(exit_aprecv), GTK_OBJECT(window));
  gtk_signal_connect_object(GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(clicked_ok), GTK_OBJECT(window));


  /* pack the table into the mainwindow */
  gtk_container_add(GTK_CONTAINER(window), table1);

  /* main ending */
  gtk_widget_show_all(window);
  gtk_main();
  return(0);
}
