#include "../include/apsrlib.h"

#ifdef HAVE_APSRLIB_TCL_SUPPORT
#define APSR_SCR_MEM_VAR	65535
#define APSR_SCR_MEM_INT	1024


/* Initialize the TCL interpreter */
Tcl_Interp * apsr_scr_init (void) {
	return(Tcl_CreateInterp());
}


/* Load script file */
int apsr_scr_load_file (Tcl_Interp *scr_intp, char *file) {
	int ret;

	ret = Tcl_EvalFile (scr_intp, file);

	if(ret != TCL_OK) {
		fprintf(stderr, "Error: %s\n", apsr_scr_get_error(scr_intp));
		exit(EXIT_FAILURE);
	}

	return(RET_SUCCESS);
}


/* Call script function */
int apsr_scr_call_func (Tcl_Interp *scr_intp, char *function) {
	int ret;

	ret = Tcl_Eval(scr_intp, function);

	if(ret != TCL_OK) {
		fprintf(stderr, "Error: %s\n", apsr_scr_get_error(scr_intp));
		exit(EXIT_FAILURE);
	}

	return(RET_SUCCESS);
}


/* Set script variable (string) */
int apsr_scr_set_var_str (Tcl_Interp *scr_intp, char *name, char *value) {
	int ret;
	char *mem;				/* Temporary memory */

	mem = xmalloc(APSR_SCR_MEM_VAR);
	snprintf(mem, APSR_SCR_MEM_VAR, "set %s {%s}", name, value);

	ret = Tcl_Eval(scr_intp, mem);
	free(mem);				/* Free the memory */

	if(ret != TCL_OK) {
		fprintf(stderr, "Warning: %s\n", apsr_scr_get_error(scr_intp));
		return(RET_FAILURE);
	} else {
		return(RET_SUCCESS);
	}
}


/* Set script variable (integer) */
int apsr_scr_set_var_int (Tcl_Interp *scr_intp, char *name, int value) {
	int ret;
	char *mem;				/* Temporary memory */

	mem = xmalloc(APSR_SCR_MEM_INT);
	snprintf(mem, APSR_SCR_MEM_INT, "set %s %d", name, value);
	
	ret = Tcl_Eval(scr_intp, mem);
	free(mem);				/* Free memory */

	if(ret != TCL_OK) {
		fprintf(stderr, "Warning: %s\n", apsr_scr_get_error(scr_intp));
		return(RET_FAILURE);
	} else {
		return(RET_SUCCESS);
	}
}


/* Get script variable (string) */
char * apsr_scr_get_var_str (Tcl_Interp *scr_intp, char *name) {
	char *value;

	value = (char *) Tcl_GetVar(scr_intp, name, 0);

	if(value == NULL) {
		fprintf(stderr, "Error: %s\n", apsr_scr_get_error(scr_intp));
		exit(EXIT_FAILURE);
	}

	return(value);
}


/* Get script variable (integer) */
int apsr_scr_get_var_int (Tcl_Interp *scr_intp, char *name) {
	return(atoi(apsr_scr_get_var_str(scr_intp, name)));
}


/* Get detailed error of the interpreter */
char * apsr_scr_get_error (Tcl_Interp *scr_intp) {
	char *ret;

	ret = (char *) Tcl_GetVar (scr_intp, "errorInfo", TCL_GLOBAL_ONLY);

        return(ret);
}
#endif

